#ifndef __EDIT_H__
#define __EDIT_H__

#ifndef IMF_AUTOFONTSIZEADJUST
 #define IMF_AUTOFONTSIZEADJUST  0x0010
#endif

//Code pages
#define KOI8_R              20866
#define CP_UNICODE_UCS2_LE -17
#define CP_UNICODE_UCS2_BE -18
#define CP_UNICODE_UTF8    -19

#define RECENTFILES 10

#define READBUFFERSIZE 8192 //Setting this value too low will lead to errors (at least 16 should be enough)
//READBUFFERSIZE _must_ be even!

//Translated text can be maximum 3 times greater (if it is translated into UTF-8)
#define TRANSLATEBUFFERSIZE (READBUFFERSIZE*3+4)
#define AUTODETECTSIZE 8192 //should not be greater than READBUFFERSIZE

//Detect binary file
#define DETECTBINARY 128

#define TEMP_BUFFER_SIZE 1024
#define ARRAY_SIZE       128

//Search/Replace options
#define FR_UP           0x00000008
#define FR_BEGINNING    0x00000010
#define FR_SELECTION    0x00000020
#define FR_ESCAPESEQ    0x00000040

//Selection case
#define UPPERCASE    1
#define LOWERCASE    2
#define SENTENCECASE 3
#define TITLECASE    4
#define INVERTCASE   5

//Insert/Delete char in selection
#define CHECK_STATE                 1
#define CHAR_INSERT                 2
#define CHAR_DELETE                 3

//Delete trailing whitespaces in selection
#define CHECK_TRAILING_WHITESPACES  1
#define DELETE_TRAILING_WHITESPACES 2

void InitEditor(HWND);
void SaveDocument(HWND hWndEdit);
void OpenDocument(HWND hWndEdit);
int FindText(HWND hWndEdit);
int ReplaceText(HWND hWndEdit, BOOL all);
BOOL TranslateBackslashes(char *string);
char TranslateHexadecimalDigit(char ch);
int TranslateNewlinesToUnix(wchar_t *buffer,int iBufSize);
BOOL isInDelimiterList(wchar_t c);
BOOL FileExists(char *fname);
BOOL RestorePreOptionsFromRegistry();
BOOL RestoreOptionsFromRegistry(HWND hWndEdit);
BOOL SaveOptionsInRegistry(HWND hWnd);
BOOL RestoreLastFilePosition(HWND);  //reads list of recent files
BOOL AddLastFilePosition(HWND);      //adds file to list of recent files
BOOL SaveLastFilePosition(HWND);     //saves list of recent files
void RecentFilesMenu(HWND hWndEdit);
void LanguageMenu(HWND hWndEdit);
void SetChosenFont(HWND hWndEdit);
int SetLayout();
void RestoreLayout(int reserv_layout);
BOOL GetModify();
void SetModify(BOOL state);
void FreeMemory();
BOOL DisplayCodePage(int cp);
HMENU API_LoadMenu(HINSTANCE hLoadInstance, LPCTSTR lpMenuName);
HACCEL API_LoadAccelerators(HINSTANCE hLoadInstance, LPCTSTR lpTableName);
HCURSOR API_LoadCursor(HINSTANCE hLoadInstance, LPCTSTR lpCursorName);
HICON API_LoadIcon(HINSTANCE hLoadInstance, LPCTSTR lpIconName);
int API_LoadString(HINSTANCE hLoadInstance, UINT uID, LPTSTR lpBuffer, int nBufferMax);

void ChangeByteOrder(unsigned char *buf, unsigned int len);
void swap(unsigned char &c1, unsigned char &c2);
void AutodetectCodePage(unsigned char *pcBuffer,int iCount);
int getmaxelement(int *base,int iCount);
/* WARNING: Caller must allocate data for pcOutBuf, 6*iBufSize should be enough for everyone ;).*/
int TranslateTextToRepresent(LPBYTE pcBuff, int iBufSize, LPBYTE pcOutBuf, LPBYTE *endptr, BOOL firsttime);
int TranslateTextFromRepresent(LPBYTE pcBuff, int iBufSize, LPBYTE pcOutBuf, BOOL firsttime);

DWORD CALLBACK InputStreamCallback(DWORD dwCookie,LPBYTE pbBuff,LONG cb,LONG FAR *pcb);
DWORD CALLBACK OutputStreamCallback(DWORD dwCookie,LPBYTE pbBuff,LONG cb,LONG FAR *pcb);
BOOL CALLBACK FindAndReplaceDlgProc(HWND hDlg,UINT msg,WPARAM wParam,LPARAM lParam);
BOOL CALLBACK GoToLineDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK AboutDlgProc(HWND hDlg,UINT msg,WPARAM wParam,LPARAM lParam);
int CALLBACK MyEditWordBreakProc(LPTSTR pchText, int ichCurrent, int cch, int code);
DWORD CALLBACK PasteCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb);
BOOL CALLBACK SelectDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
BOOL CALLBACK EnumCodePagesProc(LPTSTR lpCodePageString);

BOOL DoFileNew(HWND hWndEdit);
void DoFileNewWindow(HWND hWndEdit);
void DoFileOpen(HWND hWndEdit);
void DoFileReopen(HWND hWndEdit);
BOOL DoFileSave(HWND hWndEdit);
BOOL DoFileSaveAs(HWND hWndEdit);
void DoFilePageSetup(HWND hWndEdit);
void DoFilePrint(HWND hWndEdit);
void DoFileExit(HWND hWndEdit);

void DoEditUndo(HWND hWndEdit);
void DoEditRedo(HWND hWndEdit);
void DoEditCut(HWND hWndEdit);
void DoEditCopy(HWND hWndEdit);
void DoEditPaste(HWND hWndEdit);
void DoEditClear(HWND hWndEdit);
void DoEditSelectAll(HWND hWndEdit);
void DoEditInsertChar(HWND hWndEdit);
BOOL DoEditInsertCharInSelection(HWND hWndEdit, int nAction, char ch);
BOOL DoEditDeleteTrailingWhitespaces(HWND hWndEdit, int nAction);
void DoEditChangeCase(HWND hWndEdit, int nCase);
void DoEditWrap(HWND hWndEdit);
void DoEditKeepSpace(HWND hWndEdit);

void DoSearchFind(HWND hWndEdit);
void DoSearchFindNextDown(HWND hWndEdit);
void DoSearchFindNextUp(HWND hWndEdit);
void DoSearchReplace(HWND hWndEdit);
void DoSearchGoToLine(HWND hWndEdit);

int DoSelectCodePage(HWND hWndEdit);

void DoSettingsFont(HWND hWndEdit);
void DoSettingsBG(HWND hWndEdit);
void DoSettingsSaveTime(HWND hWndEdit);
void DoSettingsNoMultiOpen(HWND hWndEdit);
void DoSettingsOnTop(HWND hWndEdit);

BOOL DoCodePageOpenAs(HWND hWndEdit, int nCP);
BOOL DoCodePageSaveIn(HWND hWndEdit, int nCP);

void DoHelpAbout(HWND hWndEdit);

void DoNonMenuDelLine(HWND hWndEdit);

#endif
